local i18n = QBL.i18n[QBL.locale].lib.Loot

local get_id = QBL.util.Links.get_id
local fmt_notice = QBL.util.Text.fmt_notice
local is_auto_ok = QBL.util.Config.is_auto_ok
local is_inv_full = QBL.util.Trash.is_inv_full
local find_lowest_trash = QBL.util.Trash.find_lowest_trash

-- Enabled by LOOT_OPENED in this file and disabled by LOOT_CLOSED in lib/Bags.
if QBL.var.is_looting == nil then QBL.var.is_looting = false end

-- Compares 'item' value against lowest value trash. If 'item' is higher,
-- it discards the trash and loots the 'item'.
local function loot_highest(item)
	local link, bag, slot, trash_price = find_lowest_trash()
	local id = get_id(GetLootSlotLink(item))
	if trash_price and id then
		local loot_price = select(11, GetItemInfo(id))
		if loot_price and loot_price > trash_price then
			QBL:print(fmt_notice(i18n.REPLACE_TRASH, "WARN"), link)
			PickupContainerItem(bag, slot)
			DeleteCursorItem()
			LootSlot(item)
		end
	end
end

-- When inventory is full and a loot 'itm' is stackable, this function
-- tries to fit the item into an existing stack. Failing that, it determines
-- if it is more profitable to drop enough of the existing stack to take the
-- loot stack or drop the lowest trash item to take it. If it isn't, it Will
-- drop part of the bag stack to form a complete stack.
local function try_stack(itm, itm_link, itm_id, itm_count)
	-- Find compatible stacks.
	local stacks = {}
	for bag = BACKPACK_CONTAINER, NUM_BAG_SLOTS do
		for slot = 1, GetContainerNumSlots(bag) do
			local _, bag_count, _, _, _, _, link = GetContainerItemInfo(bag, slot)
			if bag_count then
				local id = get_id(link)
				if id and id == itm_id then
					local bag_stack = select(8, GetItemInfo(id))
					if bag_stack
						and bag_stack > bag_count
						and ((bag_stack - bag_count) >= itm_count)
					then
						return LootSlot(itm)
					elseif bag_stack and bag_stack > bag_count then
						table.insert(stacks, {bag = bag, slot = slot,
							free = bag_stack - bag_count})
					end
				end
			end
		end
	end

	-- Find the stack with the most space.
	local shortstack
	if #stacks > 1 then
		for first = 1, #stacks do
			for next = 1, #stacks do
				if first ~= next then
					if stacks[first].free > stacks[next].free then
						if not shortstack then shortstack = stacks[first]
						else
							if stacks[first].free > shortstack.free then
								shortstack = stacks[first]
							end
						end
					elseif stacks[first].free < stacks[next].free then
						if not shortstack then shortstack = stacks[next]
						else
							if stacks[next].free > shortstack.free then
								shortstack = stacks[next]
							end
						end
					else -- Equal.
						if not shortstack then shortstack = stacks[first]
						else
							if stacks[first].free > shortstack.free then
								shortstack = stacks[first]
							end
						end
					end
				end
			end
		end
	elseif #stacks == 1 then shortstack = stacks[1] end

	if shortstack then -- Try to split & stack.
		local drop = itm_count - shortstack.free
		local itm_price = select(11, GetItemInfo(itm_link))
		local trash_link, trash_bag, trash_slot, trash_price = find_lowest_trash()

		if trash_price and (drop * itm_price) < trash_price then
			QBL:print(fmt_notice(i18n.SPLIT_STACK, "WARN"), itm_link)
			SplitContainerItem(shortstack.bag, shortstack.slot, drop)
			DeleteCursorItem()
			LootSlot(itm)
		elseif trash_price and (itm_count * itm_price) > trash_price then
			QBL:print(fmt_notice(i18n.REPLACE_TRASH, "WARN"), trash_link)
			PickupContainerItem(trash_bag, trash_slot)
			DeleteCursorItem()
			LootSlot(itm)
		else
			QBL:print(fmt_notice(i18n.SPLIT_STACK, "WARN"), itm_link)
			SplitContainerItem(shortstack.bag, shortstack.slot, drop)
			DeleteCursorItem()
			LootSlot(itm)
		end
	end
end

-- Always loot money and quest items.
-- Will loot items above an fast_loot quality threshold, if enabled.
-- Will drop trash if loot is worth more, if enabled and nessesary.
function QBL.events:LOOT_OPENED(auto, ...)
	QBL.var.is_looting = true
	if auto == 0 then
		local full, count
		if QBL_cfg.drop_trash_loot and QBL_cfg.fast_loot > 1
		then full, count = is_inv_full() end

		for i = 1, GetNumLootItems() do
			local _, _, quantity, quality, _, quest_item = GetLootSlotInfo(i)
			if (quantity and quantity == 0) -- Money.
				or (not full and quest_item) -- Quest Item.
				or (not full -- Fast Loot filter.
					and QBL_cfg.fast_loot > 1
					and quality
					-- fast_loot quality starts from 2, system quality from 0.
					-- Subtract to compare.
					and quality >= (QBL_cfg.fast_loot - 2))
			then
				LootSlot(i)
				if QBL_cfg.drop_trash_loot and QBL_cfg.fast_loot > 1 then
					count = count - 1
					if count <= 0 then full = true end
				end

			elseif full and quest_item then
				local link = GetLootSlotLink(i)
				if link then
					local id = get_id(link)
					local _, _, count = GetLootSlotInfo(i)
					local stack = select(8, GetItemInfo(id))
					if stack and count and stack > 1 and count >= 1
					then LootSlot(i)
					else
						local trash_link, trash_bag, trash_slot = find_lowest_trash()
						if trash_bag and trash_slot then
							QBL:print(fmt_notice(i18n.REPLACE_TRASH, "WARN"), trash_link)
							PickupContainerItem(trash_bag, trash_slot)
							DeleteCursorItem()
						end
						LootSlot(i)
					end
				end

			elseif full
				and QBL_cfg.fast_loot > 1
				and quality
				and quality >= (QBL_cfg.fast_loot - 2)
			then
				local link = GetLootSlotLink(i)
				if link then
					local id = get_id(link)
					local _, _, count = GetLootSlotInfo(i)
					local stack = select(8, GetItemInfo(id))
					if stack and count and stack > 1 and count >= 1
					then try_stack(i, link, id, count)
					else loot_highest(i) end
				end
			end
		end
		if QBL_cfg.fast_loot > 1 and is_auto_ok() then CloseLoot() end
	end
end

function QBL.hook.loot_left(enable, frame)
	if enable then SetInsertItemsLeftToRight(true)
	else SetInsertItemsLeftToRight(false) end
end

QBL.startup["LOOT_OPENED"] = true
